/**
 * Insere atividade para cada item de trabalho.
 * 
 * @author Ederson Melquades da Cunha <ederson@teknisa.com>
 *
 * Objetivo: 
 *  - Incluir registro na tabela COT_ATIVIDADE a partir da tabela itemtrab 
 *  - S realizado para registros da ITEMTRAB:
 *     - cujo cdigo de arvore comece em '11' (SUBSTR(ITEMTRAB.CDARVITTRAB, 1, 2) = '11')
 *     - cujo tamanho do campo ITEMTRAB.CDREDUITTRAB = 8 (LENGTH(LTRIM(RTRIM(CDREDUITTRAB))) = 8)
 *     - sejam de sistemas web ativos, ou seja, cujo nome do sistema comece com 'WEB_' e IDCOMERCIAL = 's'.
 *       verificar se existe sistema com cdigo = 'WEB_'||SUBSTR(CDREDUITTRAB, 1, 3)
 *       exemplo:
 *           CDREDUITTRAB = 'ADM10000'
 *           verificar se existe o sistema 'WEB_ADM' (ADM = SUBSTR(CDREDUITTRAB, 1, 3)
 *           e se o mesmo est ativo (IDCOMERCIAL = 'S')
 * 
 */

CREATE OR REPLACE 
TRIGGER INSERE_ATIVIDADE_ITEMTRABALHO
AFTER INSERT OR UPDATE OR DELETE
ON ITEMTRAB 
FOR EACH ROW
DECLARE
   V_COUNT_SISTEMA NUMBER;
   V_COUNT_ATIVIDADE NUMBER; 
   V_CDSISTEMA SISTEMA.CDSISTEMA%TYPE;
   V_COT_ROTINA COT_ROTINA.NRROTINA%TYPE;
   V_COT_ATIVIDADE NOVOCODIGO.NRSEQUENCIAL%TYPE;
BEGIN
   IF    DELETING 
      OR (SUBSTR(:NEW.CDARVITTRAB, 1, 2) <> '11') 
      OR (LENGTH(LTRIM(RTRIM(:NEW.CDREDUITTRAB))) <> 8) THEN
      
      SELECT COUNT(*) 
        INTO V_COUNT_ATIVIDADE 
        FROM COT_ATIVIDADE 
       WHERE CDINTEGRACAO = 'DELPHI_'||:OLD.CDREDUITTRAB 
         AND NRORG = 1;
     
      IF V_COUNT_ATIVIDADE <> 0 THEN
         IF DELETING THEN 
            UPDATE COT_ATIVIDADE
               SET CDINTEGRACAO = 'DELPHI_'||:OLD.CDREDUITTRAB||' (item de trabalho excludo)',
                   IDATIVO = 'N'
             WHERE CDINTEGRACAO = 'DELPHI_'||:OLD.CDREDUITTRAB
               AND NRORG = 1;
         END IF;   
         IF (SUBSTR(:NEW.CDARVITTRAB, 1, 2) <> '11') THEN -- ITEM SAIU DA PASTA "11 -> 1 Cdigo Fonte / 1 Sistemas Teknisa"
            UPDATE COT_ATIVIDADE
               SET CDINTEGRACAO = 'DELPHI_'||:OLD.CDREDUITTRAB||' (item movido para outra pasta: '||:NEW.CDARVITTRAB||')',
                   IDATIVO = 'N'
             WHERE CDINTEGRACAO = 'DELPHI_'||:OLD.CDREDUITTRAB
               AND NRORG = 1;            
         END IF;
         IF (LENGTH(LTRIM(RTRIM(:NEW.CDREDUITTRAB))) <> 8) THEN --SE TAMANHO DO CAMPO CDREDUITTRAB <> 8, ALTERAR O CAMPO CDINTEGRACAO E INATIVAR O REGISTRO 
            UPDATE COT_ATIVIDADE
               SET CDINTEGRACAO = 'DELPHI_'||:OLD.CDREDUITTRAB||' (cdigo alterado com tamanho invlido: '||:NEW.CDREDUITTRAB||')',
                   IDATIVO = 'N'
             WHERE CDINTEGRACAO = 'DELPHI_'||:OLD.CDREDUITTRAB
               AND NRORG = 1;
         END IF;   
      END IF;
   END IF;

   -- PARA ITENS DE TRABALHO COM CDIGO DE ARVORE QUE COMECE EM '11', OU SEJA, ESTEJA NA PASTA "11 -> 1 Cdigo Fonte / 1 Sistemas Teknisa"
   -- E O CDIGO REDUZIDO COM TAMANHO = 8 
   IF INSERTING OR UPDATING THEN
   
      IF     (SUBSTR(:NEW.CDARVITTRAB, 1, 2) = '11') 
         AND (LENGTH(LTRIM(RTRIM(:NEW.CDREDUITTRAB))) = 8)  
         AND (:NEW.NRORG = 1) THEN
      
         --PARA SISTEMAS WEB ATIVOS, OU SEJA, CUJO NOME DO SISTEMA COMECE COM 'WEB_' E IDCOMERCIAL = 'S' 
         SELECT COUNT(*) 
           INTO V_COUNT_SISTEMA 
           FROM SISTEMA 
          WHERE CDSISTEMA = 'WEB_'||SUBSTR(:NEW.CDREDUITTRAB, 1, 3) 
            AND IDCOMERCIAL = 'S' 
            AND NRORG = 1;

         BEGIN
            SELECT CDSISTEMA 
              INTO V_CDSISTEMA 
              FROM SISTEMA 
             WHERE CDSISTEMA = 'WEB_'||SUBSTR(:NEW.CDREDUITTRAB, 1, 3) 
               AND IDCOMERCIAL = 'S' 
               AND NRORG = 1;
         EXCEPTION
            WHEN OTHERS THEN V_CDSISTEMA := NULL;
         END;

        --LOCALIZA QUAL O PROCESSO DEFAULT DO SISTEMA CORRESPONDENTE, DEPOIS QUAL A ROTINA DEFAULT CORRESPONDENTE DA ATIVIDADE
         BEGIN
            SELECT ROT.NRROTINA
              INTO V_COT_ROTINA
              FROM COT_PROCESSO PRO, COT_ROTINA ROT
             WHERE PRO.CDSISTEMA = V_CDSISTEMA
               AND PRO.IDDEFAULT = 'S'
               AND PRO.NRORG = 1
               AND ROT.NRPROCESSOCOT = PRO.NRPROCESSOCOT 
               AND ROT.NRORG = PRO.NRORG 
               AND ROT.IDDEFAULT = 'S';
         EXCEPTION
            WHEN OTHERS THEN V_COT_ROTINA := NULL;
         END;

         IF     INSERTING 
            AND (V_COUNT_SISTEMA <> 0) 
            AND (V_COT_ROTINA IS NOT NULL) THEN
            NOVO_CODIGO_SC('COT_ATIVIDADE', V_COT_ATIVIDADE, 1, 1);
            INSERT INTO COT_ATIVIDADE
               (NRATIVIDADE, NRROTINA, NMATIVIDADE, 
                DSATIVIDADE, NRORDENACAO, CDATIVIDADE, 
                CDINTEGRACAO, IDPADRAOMATRIZCLIE, IDPADRAOFILIALCLIE,
                IDPADRAOMATRIZPROS, IDPADRAOFILIALPROS, QTHRATIVIDADE)
            VALUES
               (V_COT_ATIVIDADE, V_COT_ROTINA, :NEW.NMITEMTRAB, 
                NULL, 1, :NEW.CDREDUITTRAB, 
                'DELPHI_'||:NEW.CDREDUITTRAB, 'N', 'N', 
                'N', 'N', 0); 
         END IF;  

         IF UPDATING THEN     
            --LOCALIZA A ATIVIDADE CORRESPONDENTE ATRAVS DO CAMPO CDINTEGRAO
            SELECT COUNT(*)
              INTO V_COUNT_ATIVIDADE
              FROM COT_ATIVIDADE
             WHERE CDINTEGRACAO = 'DELPHI_'||:OLD.CDREDUITTRAB
               AND NRORG = 1;
            
            --ALTERAR O NOME DA ATIVIDADE CORRESPONDENTE COM O NOME DO ITEM DE TRABALHO ALTERADO 
            IF     (V_COUNT_ATIVIDADE <> 0) 
               AND (:NEW.NMITEMTRAB <> :OLD.NMITEMTRAB) THEN   
               UPDATE COT_ATIVIDADE
                  SET NMATIVIDADE = :NEW.NMITEMTRAB
                WHERE CDINTEGRACAO = 'DELPHI_'||:OLD.CDREDUITTRAB
                  AND NRORG = 1;
            END IF;
            
           --QUANDO O CDIGO REDUZIDO DO ITEM DE TRABALHO FOR ALTERADO
            IF (:OLD.CDREDUITTRAB <> :NEW.CDREDUITTRAB) THEN
               -- SE O SISTEMA FOI ALTERADO, ALTERAR O CAMPO CDINTEGRACAO, INATIVAR O REGISTRO E 
               -- CRIAR OUTRO REGISTRO DE ATIVIDADE COM O CDIGO NOVO (CDREDUITTRAB)
               IF (SUBSTR(:NEW.CDREDUITTRAB, 1, 3) <> SUBSTR(:OLD.CDREDUITTRAB, 1, 3)) THEN
                  IF (V_COUNT_ATIVIDADE <> 0) THEN
                     UPDATE COT_ATIVIDADE
                        SET CDINTEGRACAO = 'DELPHI_'||:OLD.CDREDUITTRAB||' (Codigo alterado para outro sistema: '||:NEW.CDREDUITTRAB||' )',
                            IDATIVO = 'N'
                      WHERE CDINTEGRACAO = 'DELPHI_'||:OLD.CDREDUITTRAB
                        AND NRORG = 1;
                  END IF;
                  IF     (V_COUNT_SISTEMA <> 0) 
                     AND (V_COT_ROTINA IS NOT NULL) THEN
                     NOVO_CODIGO_SC('COT_ATIVIDADE', V_COT_ATIVIDADE, 1, 1);
                     INSERT INTO COT_ATIVIDADE
                        (NRATIVIDADE, NRROTINA, NMATIVIDADE, 
                         DSATIVIDADE, NRORDENACAO, CDATIVIDADE, 
                         CDINTEGRACAO, IDPADRAOMATRIZCLIE, IDPADRAOFILIALCLIE,
                         IDPADRAOMATRIZPROS, IDPADRAOFILIALPROS, QTHRATIVIDADE)
                     VALUES
                        (V_COT_ATIVIDADE, V_COT_ROTINA, :NEW.NMITEMTRAB, 
                         NULL, 1, :NEW.CDREDUITTRAB, 
                         'DELPHI_'||:NEW.CDREDUITTRAB, 'N', 'N', 
                         'N', 'N', 0); 
                  END IF;    
               ELSE -- CDIGO REDUZIDO FOI ALTERADO, MAS PERMANECEU NO MESMO SISTEMA
                  IF (V_COUNT_ATIVIDADE <> 0) THEN -- SE J EXISTIA ATIVIDADE PARA O ITEM DE TABALHO, UPDATA
                     UPDATE COT_ATIVIDADE
                        SET CDATIVIDADE = :NEW.CDREDUITTRAB,
                            CDINTEGRACAO = 'DELPHI_'||:NEW.CDREDUITTRAB
                      WHERE CDINTEGRACAO = 'DELPHI_'||:OLD.CDREDUITTRAB
                        AND NRORG = 1;
                  ELSE -- SE AINDA NO EXISTIA ATIVIDADE PARA O ITEM DE TRABALHO, CRIA UMA NOVA
                     IF     (V_COUNT_SISTEMA <> 0) 
                        AND (V_COT_ROTINA IS NOT NULL) THEN
                        NOVO_CODIGO_SC('COT_ATIVIDADE', V_COT_ATIVIDADE, 1, 1);
                        INSERT INTO COT_ATIVIDADE
                           (NRATIVIDADE, NRROTINA, NMATIVIDADE, 
                            DSATIVIDADE, NRORDENACAO, CDATIVIDADE, 
                            CDINTEGRACAO, IDPADRAOMATRIZCLIE, IDPADRAOFILIALCLIE,
                            IDPADRAOMATRIZPROS, IDPADRAOFILIALPROS, QTHRATIVIDADE)
                        VALUES
                           (V_COT_ATIVIDADE, V_COT_ROTINA, :NEW.NMITEMTRAB, 
                            NULL, 1, :NEW.CDREDUITTRAB, 
                            'DELPHI_'||:NEW.CDREDUITTRAB, 'N', 'N', 
                            'N', 'N', 0); 
                     END IF;       
                  END IF;
               END IF;
            ELSE
            --CASO NO EXISTA ATIVIDADE PARA O CDREDUITTRAB INFORMADO, A MESMA  CRIADA
               IF (V_COUNT_ATIVIDADE = 0) THEN          
                  IF     (V_COUNT_SISTEMA <> 0) 
                     AND (V_COT_ROTINA IS NOT NULL) THEN
                     NOVO_CODIGO_SC('COT_ATIVIDADE', V_COT_ATIVIDADE, 1, 1);
                     INSERT INTO COT_ATIVIDADE
                        (NRATIVIDADE, NRROTINA, NMATIVIDADE, 
                         DSATIVIDADE, NRORDENACAO, CDATIVIDADE, 
                         CDINTEGRACAO, IDPADRAOMATRIZCLIE, IDPADRAOFILIALCLIE,
                         IDPADRAOMATRIZPROS, IDPADRAOFILIALPROS, QTHRATIVIDADE)
                     VALUES
                        (V_COT_ATIVIDADE, V_COT_ROTINA, :NEW.NMITEMTRAB, 
                         NULL, 1, :NEW.CDREDUITTRAB, 
                         'DELPHI_'||:NEW.CDREDUITTRAB, 'N', 'N', 
                         'N', 'N', 0); 
                  END IF;       
               END IF;
            END IF;
         END IF;
      END IF;   
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('INSERE_ATIVIDADE_ITEMTRABALHO', '0.00.000'); END;
/